@@ -957,48 +957,51 @@ int ProcessAttrRqst(SLPMessage message,
     size = message->header.langtaglen + 19; /* 14 bytes for header     */
                                             /*  2 bytes for error code */
                                             /*  2 bytes for attr-list len */
-                                            /*  1 byte for the authcount */
-    size += db->attrlistlen;
-
-#ifdef ENABLE_SLPv2_SECURITY
-
-    /*------------------------------------------------------------------*/
-    /* Generate authblock if necessary or just use the one was included */
-    /* by registering agent.  Reserve sufficent space for either case.  */
-    /*------------------------------------------------------------------*/
-    if (G_SlpdProperty.securityEnabled &&
-        message->body.attrrqst.spistrlen )
+                                            /*  1 byte for the authcount */ 
+    if(errorcode == 0)
     {
-        if (message->body.attrrqst.taglistlen == 0)
+        size += db->attrlistlen;
+    
+    #ifdef ENABLE_SLPv2_SECURITY
+    
+        /*------------------------------------------------------------------*/
+        /* Generate authblock if necessary or just use the one was included */
+        /* by registering agent.  Reserve sufficent space for either case.  */
+        /*------------------------------------------------------------------*/
+        if (G_SlpdProperty.securityEnabled &&
+            message->body.attrrqst.spistrlen )
         {
-            for (i=0; i<db->authcount;i++)
+            if (message->body.attrrqst.taglistlen == 0)
             {
-                if (SLPCompareString(db->autharray[i].spistrlen,
-                                     db->autharray[i].spistr,
-                                     message->body.attrrqst.spistrlen,
-                                     message->body.attrrqst.spistr) == 0)
+                for (i=0; i<db->authcount;i++)
                 {
-                    opaqueauth = db->autharray[i].opaque;
-                    opaqueauthlen = db->autharray[i].opaquelen;
-                    break;
+                    if (SLPCompareString(db->autharray[i].spistrlen,
+                                         db->autharray[i].spistr,
+                                         message->body.attrrqst.spistrlen,
+                                         message->body.attrrqst.spistr) == 0)
+                    {
+                        opaqueauth = db->autharray[i].opaque;
+                        opaqueauthlen = db->autharray[i].opaquelen;
+                        break;
+                    }
                 }
             }
+            else
+            {
+                errorcode = SLPAuthSignString(G_SlpdSpiHandle,
+                                              message->body.attrrqst.spistrlen,
+                                              message->body.attrrqst.spistr,
+                                              db->attrlistlen,
+                                              db->attrlist,
+                                              &generatedauthlen,
+                                              &generatedauth);
+                opaqueauthlen = generatedauthlen;
+                opaqueauth = generatedauth;
+            }
+            size += opaqueauthlen;
         }
-        else
-        {
-            errorcode = SLPAuthSignString(G_SlpdSpiHandle,
-                                          message->body.attrrqst.spistrlen,
-                                          message->body.attrrqst.spistr,
-                                          db->attrlistlen,
-                                          db->attrlist,
-                                          &generatedauthlen,
-                                          &generatedauth);
-            opaqueauthlen = generatedauthlen;
-            opaqueauth = generatedauth;
-        }
-        size += opaqueauthlen;
+    #endif
     }
-#endif
 
     /*-------------------*/
     /* Alloc the  buffer */
@@ -1226,7 +1229,10 @@ int ProcessSrvTypeRqst(SLPMessage message,
                                             /*  2 bytes for error code */
                                             /*  2 bytes for srvtype
                                                 list length  */
-    size += db->srvtypelistlen;
+    if(errorcode == 0)
+    {
+        size += db->srvtypelistlen;
+    }
 
 
     /*------------------------------*/
@@ -1325,11 +1331,14 @@ int SLPDProcessMessage(struct sockaddr_in* peerinfo,
 
     SLPDLogMessage(SLPDLOG_TRACEMSG_IN,peerinfo,recvbuf);
 
-    /* set the sendbuf empty */
-    if(*sendbuf)
+    if(!*sendbuf)
     {
-        (*sendbuf)->end = (*sendbuf)->start;
+	*sendbuf = SLPBufferAlloc(SLP_MAX_DATAGRAM_SIZE);
+	if (!*sendbuf)
+	    return SLP_ERROR_PARSE_ERROR;
     }
+    /* set the sendbuf empty */
+    (*sendbuf)->end = (*sendbuf)->start;
 
     /* zero out the header before parsing it */
     memset(&header,0,sizeof(header));
